from typing import Optional

import requests
import os
import sys
import json
import semver
import subprocess


endpoint = os.environ['NETDB_ENDPOINT']

def exec_command_into_file(command: list[str], file: Optional[str], env: dict[str, str | None]):
    full_env = dict(os.environ)
    full_env.update(env)
    p = subprocess.Popen(command, env=full_env, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout, err = p.communicate()
    p.wait()
    if file is not None:
        with open(file, 'w') as f:
            f.write(stdout.decode('utf-8'))
    if len(err.decode('utf-8')) > 0:
        print(f'Warning: Captured non-empty STDERR while executing {command}')
        print(err.decode('utf-8'), file=sys.stderr)

def dump_data(main_version, obj, f_name, sort_key='name'):
	d = None
	if sort_key is not None:
		req = requests.get(f'https://{endpoint}/{main_version}/wapi/{obj}/list?sorting_params_list=["{sort_key}"]')
		if req.status_code != 200:
			print(req.text)
			assert req.status_code == 200
		d = req.json()[0]
	else:
		req = requests.get(f'https://{endpoint}/{main_version}/wapi/{obj}/list')
		if req.status_code != 200:
			print(req.text)
			assert req.status_code == 200
		d = req.json()[0]
	with open(f"{main_version}/{f_name}.json", "w") as f:
		json.dump(d, f, indent=2)

versions = requests.get(f'https://{endpoint}/').json()[0]

latest_ver = None
api_generator = '/venv/bin/net-api-generator'
for v in versions:
	ver = semver.Version.parse(v['numeric'])
	if latest_ver is None or ver > latest_ver:
		latest_ver = ver
	main_version = f"{v['major']}.{v['minor']}"
	os.mkdir(main_version)
	with open(f"{main_version}/version_numeric", "w") as f:
		f.write(v['numeric'])
	if 'transaction_json_schema' in v:
		with open(f"{main_version}/transaction_json_schema.json", "w") as f:
			json.dump(v['transaction_json_schema'], f, indent=2)
	dump_data(main_version, 'data_type', 'data_types')
	dump_data(main_version, 'function', 'functions')
	dump_data(main_version, 'object_type', 'objects')
	dump_data(main_version, 'system', 'systems')
	if v['major'] > 3 and not (v['major'] == 4 and v['minor'] == 0):
		dump_data(main_version, 'data_type_operator', 'data_type_ops')
	else:
		dump_data(main_version, 'data_type_operator', 'data_type_ops', None)
	exec_command_into_file([api_generator, 'openapi'], f'{main_version}/openapi.yml', {'NETDB_VERSION': main_version})
	os.mkdir(main_version + '/es')
	exec_command_into_file([api_generator, 'es-webpack', f'--output-dir={main_version}/es'], None, {'NETDB_VERSION': main_version})
	os.mkdir(main_version + '/python')
	exec_command_into_file([api_generator, 'python', f'--output-dir={main_version}/python', ], None, {'NETDB_VERSION': main_version})
	os.mkdir(main_version + '/golang')
	exec_command_into_file([api_generator, 'golang-structs', f'--output={main_version}/golang/structs.go'], None, {'NETDB_VERSION': main_version})

with open(f"latest_version", "w") as f:
	f.write(str(latest_ver))
